#! /bin/sh

# This is the configuration script for the Citadel/UX Makefile.  Do not attempt
# to edit any of this stuff yourself unless you really know what you are doing!


[ x$* = x ] && {
	echo This script is designed to be executed by the Makefile.
	echo There is no need to run it yourself.
	exit
	}

CC=$1

echo -------- Now generating the sysdep.h header for your system. --------

cat >sysdep.h <<!
/*
 * sysdep.h - system dependent definitions
 *
 * THIS FILE IS GENERATED BY configure.sh WHEN THE MAKEFILE IS RUN.
 * DO NOT EDIT THIS FILE BY HAND!!
 *
 */

!



##############################################################################
#                                                                            #
#                  Test to see if we have the flock() system call            #
#                                                                            #
##############################################################################

echo
echo Testing for flock system call...
cat >testing.c <<!


#include <stdio.h>
#include <fcntl.h>
#include <sys/file.h>

main() {
	int mms_fd;
	int a;

	a=flock(mms_fd,LOCK_EX);
	a=flock(mms_fd,LOCK_UN);
	exit(0);
	}


!

if $CC testing.c -o testing 2>/dev/null; then
	echo ...flock available.
	echo "#define HAVE_FLOCK" >>sysdep.h
else
	echo ...not available.
fi
rm -f testing testing.o testing.c >/dev/null 2>&1



##############################################################################
#                                                                            #
#              Test to see if we need to proto htons                         #
#                                                                            #
##############################################################################

echo
echo Testing to see if we need to prototype htons
cat >testing.c <<!

#include <stdio.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <string.h>

u_short htons();

void main() {
	exit(0);
	}
!

$CC testing.c -o testing 2>/dev/null
if $CC testing.c -o testing 2>/dev/null; then
	echo OK to prototype it.
else
	echo Not OK to prototype it, suppressing...
	echo "#define DONT_PROTO_HTONS" >>sysdep.h
fi
rm -f testing testing.o testing.c >/dev/null 2>&1




##############################################################################
#                                                                            #
#              Test to see if we have strerror                               #
#                                                                            #
##############################################################################

echo
echo Checking for strerror function
cat >testing.c <<!

#include <string.h>

main() {
	printf("%s\n",strerror(1));
	}

!

$CC testing.c -o testing 2>/dev/null
if $CC testing.c -o testing 2>/dev/null; then
	echo This system has it.
else
	echo This system does not have it - using a substitute
	echo "#define NO_STRERROR" >>sysdep.h
fi
rm -f testing testing.o testing.c >/dev/null 2>&1



##############################################################################
#                                                                            #
#              Test to see if we have gethostbyaddr()                        #
#                                                                            #
##############################################################################

echo
echo Checking for gethostbyaddr etc.
cat >testing.c <<!

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <string.h>

void main() {
	struct sockaddr_in      cs;     
	struct hostent      *   ch;        
	int                     len;     
	int			a;
	char *i;
    len = sizeof(cs);   
    if (getpeername(0,(struct sockaddr *)&cs,&len) < 0) {   
	return;
	}
    if((ch = gethostbyaddr((char *) &cs.sin_addr, sizeof(cs.sin_addr),  
	  AF_INET)) == NULL) { 
		return;
		}
	}

!

$CC testing.c -o testing 2>/dev/null
if $CC testing.c -o testing 2>/dev/null; then
	echo This system has these functions available.
	echo "#define HOST_OK" >>sysdep.h
else
	echo Not available ... assuming all connections are local.
fi
rm -f testing testing.o testing.c >/dev/null 2>&1



##############################################################################
#                                                                            #
#           Do we create named pipes with mknod or mkfifo?                   #
#                                                                            #
##############################################################################

echo
echo Checking for mkfifo command...
echo
if mkfifo /tmp/citconf$$; then
	echo This system has a mkfifo command.
	echo "#define USE_MKFIFO" >>sysdep.h
else
	echo This system does not have mkfifo, using mknod instead.
	fi
rm -f /tmp/citconf$$ 2>/dev/null
